from google.cloud import logging
from retrying import retry
import ee_manager.config as config
import socket, errno


#
# WRITE TO LOGS
#
def safe_log(trace_info='ee_manager.logger.safe_log',**kwargs):
    try:
        log(**kwargs)
        return True
    except socket.error, e:
        return _handle_socket_error(e,'ee_manager.safe_log ({})'.format(trace_info))
    except Exception as e: 
        msg="\n\n(ee-manager-logger.log) ERROR[{}]: {}\n\t{}\n".format(trace_info,e,kwargs)
        print msg
        return False
    except:
        msg="\n\n(ee-manager-logger.log) BREAK[{}]\n\t{}\n".format(trace_info,kwargs)
        print msg
        return None


@retry(wait_exponential_multiplier=config.WAIT_EXP_MULTIPLIER, wait_exponential_max=config.WAIT_EXP_MAX, stop_max_delay=config.STOP_MAX_DELAY)
def log(text=None,data=None,severity=config.DEFAULT_LOG_SEVERITY,name=config.DEFAULT_LOG_NAME,**kwargs):
    """ return None
    triggers text and/or sturct log to stackdriver.
        - text<str>
        - data<dict>
        - severity<str> 
            one of the following:
            * DEFAULT
            * DEBUG
            * INFO
            * NOTICE
            * WARNING
            * ERROR
            * CRITICALALERT
            * EMERGENCY
    """
    print "\t(ee-manager-logging)[{}] text: {}  data: {}  kwargs: {}".format(severity,text,data,kwargs)
    logger = logging.Client().logger(name)
    if text: logger.log_text(text,severity=severity)
    if data: logger.log_struct(data,severity=severity)    
    if kwargs: logger.log_struct(kwargs,severity=severity)
    print "\t(ee-manager-logging)----------------|"


#
#   INTERNAL
#
def _handle_socket_error(e,trace_info='logger.handle_socket_error'):
    if isinstance(e.args, tuple):
        print "\n\n\t\terrno is %d" % e[0]
        if e[0] == errno.EPIPE:
           # remote peer disconnected
           print "\t\tremote disconnect"
        else:
           # determine and handle different error
           print "\t\tother"
        print e,"\n\n"
    else:
        print "\n\nsocket error ", e,"\n\n"
    return False