from retrying import retry
import ee
import ee_manager.utils as utils
from ee_manager.mock_ee_task import MockEETask


"""
	EETask:
		gee task helpers to be used with EEBatchListner
"""

WAIT_EXP_MULTIPLIER=4000
WAIT_EXP_MAX=20000
STOP_MAX_DELAY=120000

class Task(object):
	# config
	TESTING=False
	# states
	ACTIVE_STATES=['READY','RUNNING']
	FINISHED_STATES=['COMPLETED','FAILED','CANCELLED','CANCEL_REQUESTED']
	CANCELLED_STATES=['CANCELLED','CANCEL_REQUESTED']
	# properties
	id=None
	index=None
	description=None
	data=None
	max_wait=None
	min_wait=None
	callback=None
	success_callback=None
	error_callback=None
	state=None
	start_timestamp=None
	creation_timestamp=None
	update_timestamp=None
	status=None
	callback_module=None
	testing=None


	#
	#	PUBLIC METHODS
	#
	def __init__(self,id,description=None,data=None,success_callback=None,error_callback=None,callback_module=None,max_wait=None,min_wait=None,testing=TESTING):
		self.id=id
		self.description=description
		self.data=data
		self.max_wait=max_wait
		self.min_wait=min_wait
		self.success_callback=success_callback
		self.error_callback=error_callback
		self.set_callback_module(callback_module)
		self.testing=testing


	def set_callback_module(self,callback_module):
		if callback_module: self.callback_module=callback_module


	def update_status(self,task_dict=None):
		if not task_dict: task_dict=utils.safe_exec(self._get_status)
		if task_dict:
			self.state=task_dict.get('state')
			self.creation_timestamp=task_dict.get('creation_timestamp_ms')
			self.start_timestamp=task_dict.get('start_timestamp_ms')
			self.update_timestamp=task_dict.get('update_timestamp_ms')
			self.status=task_dict
			return self.status


	def runtime(self):
		if self.update_timestamp and self.start_timestamp:
			return max((self.update_timestamp-self.start_timestamp),0)
		else:
			return 0


	def finalize(self):
		if self.complete() and self.success_callback:
			return utils.exec_function(
				self.success_callback,
				self.callback_module,
				state=self.state,
				task_id=self.id,
				description=self.description,
				data=self.data,
				status=self.status)
		elif self.failed() and self.error_callback:
			return utils.exec_function(
				self.error_callback,
				self.callback_module,
				state=self.state,
				task_id=self.id,
				description=self.description,
				data=self.data,
				status=self.status)
		else: return True


	""" state helpers """
	def complete(self):
		return (self.state=='COMPLETED')


	def failed(self):
		return (self.state=='FAILED')


	def unsubmitted(self):
		return (self.state=='UNSUBMITTED')


	def cancelled(self):
		return (self.state in self.CANCELLED_STATES)


	def active(self):
		return (self.state in self.ACTIVE_STATES)


	def finished(self):
		return (self.state in self.FINISHED_STATES)



	#
	# INTERNAL METHODS
	#
	@retry(wait_exponential_multiplier=WAIT_EXP_MULTIPLIER, wait_exponential_max=WAIT_EXP_MAX, stop_max_delay=STOP_MAX_DELAY)
	def _get_status(self):
		if not self.testing:
			status=ee.data.getTaskStatus(self.id)
			if status: return status[0]
		else:
			return MockEETask(self.id).status()




