import os, time
import config
import ee_manager.logger as logger
import socket, errno

#
# EXECUTE FUNCTION BY NAME
#
def _get_function(func_name,func_scope):
    func=None
    if func_scope and hasattr(func_scope,func_name):
        func=getattr(func_scope,func_name)
    if not func:
        func=globals().get(func_name)
    return func


def exec_function(func,func_scope=None,safe=True,**kwargs):
    if type(func) is str: 
        func=_get_function(func,func_scope)
    if safe:
        if safe: return safe_exec(func,**kwargs)
        else: return func(**kwargs)
    else: return False


def safe_exec(func,**kwargs):
    """ return function-call/False/None
    
    Execute function safely:
    - func: function that takes no arguments or only kwargs
    - return function() if no exception is thrown
    - return False for exceptions
    - return None for breaks
    """
    try:
        return func(**kwargs)
    except socket.error, e:
        return _handle_socket_error(e,'ee_manager.safe_exec ({})'.format(func.__name__))
    except Exception as e: 
        trace=_obj_trace(func)
        msg="(ee-manager) ERROR[{}]: {}".format(trace,e)
        logger.safe_log(text=msg,severity='ERROR',trace_info=trace)
        print msg
        return False
    except:
        trace=_obj_trace(func)
        msg="(ee-manager) BREAK[{}]".format(trace)
        logger.safe_log(text=msg,severity='WARNING',trace_info=trace)
        print msg
        return None


def _obj_trace(obj):
    if hasattr(obj, '__name__'): return obj.__name__
    else: return "{}".format(obj)


def wait(wait_time,trace_info='ee_manager.utils'):
    """ return <bool>/None
    
    Execute time.sleep safely
    """
    try:
        time.sleep(wait_time)
        return True
    except socket.error, e:
        return _handle_socket_error(e,'ee_manager.safe_exec ({})'.format(trace_info))
    except Exception as e: 
        msg="(ee-manager-utils.wait) ERROR[{}]: {}".format(trace_info,e)
        logger.safe_log(text=msg,severity='ERROR',trace_info=trace_info)
        print msg
        return False
    except:
        msg="(ee-manager-utils.wait) BREAK[{}]".format(trace_info)
        logger.safe_log(text=msg,severity='WARNING',trace_info=trace_info)
        print msg
        return None


def set_timezone(tz=config.TIMEZONE):
    os.environ['TZ']=tz
    time.tzset()


def timenow():
    return "{}".format(time.strftime('%H:%M:%S  %Z'))

#
#   INTERNAL
#
def _handle_socket_error(e,trace_info='utils._handle_socket_error'):
    if isinstance(e.args, tuple):
        print "\n\n\t\terrno is %d" % e[0]
        if e[0] == errno.EPIPE:
           # remote peer disconnected
           print "\t\tremote disconnect"
        else:
           # determine and handle different error
           print "\t\tother"
        print e,"\n\n"
    else:
        print "\n\nsocket error ", e,"\n\n"
    return False


